;;; win32-win.el --- parse switches controlling interface with win32
;; Copyright (C) 1993, 1994 Free Software Foundation, Inc.

;; Author: Kevin Gallo
;; Keywords: terminals

;;; This file is part of GNU Emacs.
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; win32-win.el:  this file is loaded from ../lisp/startup.el when it recognizes
;; that win32 windows are to be used.  Command line switches are parsed and those
;; pertaining to win32 are processed and removed from the command line.  The
;; win32 display is opened and hooks are set for popping up the initial window.

;; startup.el will then examine startup files, and eventually call the hooks
;; which create the first window (s).

;;; Code:


;; These are the standard X switches from the Xt Initialize.c file of
;; Release 4.

;; Command line		Resource Manager string

;; +rv			*reverseVideo
;; +synchronous		*synchronous
;; -background		*background
;; -bd			*borderColor
;; -bg			*background
;; -bordercolor		*borderColor
;; -borderwidth		.borderWidth
;; -bw			.borderWidth
;; -display		.display
;; -fg			*foreground
;; -fn			*font
;; -font		*font
;; -foreground		*foreground
;; -geometry		.geometry
;; -i			.iconType
;; -itype		.iconType
;; -iconic		.iconic
;; -name		.name
;; -reverse		*reverseVideo
;; -rv			*reverseVideo
;; -selectionTimeout    .selectionTimeout
;; -synchronous		*synchronous
;; -xrm

;; An alist of X options and the function which handles them.  See
;; ../startup.el.

(if (not (eq window-system 'win32))
    (error "%s: Loading win32-win.el but not compiled for win32" (invocation-name)))
	 
(require 'frame)
(require 'mouse)
(require 'scroll-bar)
(require 'faces)
(require 'select)
(require 'menu-bar)

(defvar x-invocation-args)

(defvar x-command-line-resources nil)

(defconst x-option-alist
  '(("-bw" .	x-handle-numeric-switch)
    ("-d" .		x-handle-display)
    ("-display" .	x-handle-display)
    ("-name" .	x-handle-name-rn-switch)
    ("-rn" .	x-handle-name-rn-switch)
    ("-T" .		x-handle-switch)
    ("-r" .		x-handle-switch)
    ("-rv" .	x-handle-switch)
    ("-reverse" .	x-handle-switch)
    ("-fn" .	x-handle-switch)
    ("-font" .	x-handle-switch)
    ("-ib" .	x-handle-numeric-switch)
    ("-g" .		x-handle-geometry)
    ("-geometry" .	x-handle-geometry)
    ("-fg" .	x-handle-switch)
    ("-foreground".	x-handle-switch)
    ("-bg" .	x-handle-switch)
    ("-background".	x-handle-switch)
    ("-ms" .	x-handle-switch)
    ("-itype" .	x-handle-switch)
    ("-i" 	.	x-handle-switch)
    ("-iconic" .	x-handle-iconic)
    ("-xrm" .       x-handle-xrm-switch)
    ("-cr" .	x-handle-switch)
    ("-vb" .	x-handle-switch)
    ("-hb" .	x-handle-switch)
    ("-bd" .	x-handle-switch)))

(defconst x-long-option-alist
  '(("--border-width" .	"-bw")
    ("--display" .	"-d")
    ("--name" .		"-name")
    ("--title" .	"-T")
    ("--reverse-video" . "-reverse")
    ("--font" .		"-font")
    ("--internal-border" . "-ib")
    ("--geometry" .	"-geometry")
    ("--foreground-color" . "-fg")
    ("--background-color" . "-bg")
    ("--mouse-color" .	"-ms")
    ("--icon-type" .	"-itype")
    ("--iconic" .	"-iconic")
    ("--xrm" .		"-xrm")
    ("--cursor-color" .	"-cr")
    ("--vertical-scroll-bars" . "-vb")
    ("--border-color" .	"-bd")))

(defconst x-switch-definitions
  '(("-name" name)
    ("-T" name)
    ("-r" reverse t)
    ("-rv" reverse t)
    ("-reverse" reverse t)
    ("-fn" font)
    ("-font" font)
    ("-ib" internal-border-width)
    ("-fg" foreground-color)
    ("-foreground" foreground-color)
    ("-bg" background-color)
    ("-background" background-color)
    ("-ms" mouse-color)
    ("-cr" cursor-color)
    ("-itype" icon-type t)
    ("-i" icon-type t)
    ("-vb" vertical-scroll-bars t)
    ("-hb" horizontal-scroll-bars t)
    ("-bd" border-color)
    ("-bw" border-width)))

;; Handler for switches of the form "-switch value" or "-switch".
(defun x-handle-switch (switch)
  (let ((aelt (assoc switch x-switch-definitions)))
    (if aelt
	(if (nth 2 aelt)
	    (setq default-frame-alist
		  (cons (cons (nth 1 aelt) (nth 2 aelt))
			default-frame-alist))
	  (setq default-frame-alist
		(cons (cons (nth 1 aelt)
			    (car x-invocation-args))
		      default-frame-alist)
		x-invocation-args (cdr x-invocation-args))))))

;; Make -iconic apply only to the initial frame!
(defun x-handle-iconic (switch)
  (setq initial-frame-alist
	(cons '(visibility . icon) initial-frame-alist)))

;; Handler for switches of the form "-switch n"
(defun x-handle-numeric-switch (switch)
  (let ((aelt (assoc switch x-switch-definitions)))
    (if aelt
	(setq default-frame-alist
	      (cons (cons (nth 1 aelt)
			  (string-to-int (car x-invocation-args)))
		    default-frame-alist)
	      x-invocation-args
	      (cdr x-invocation-args)))))

;; Handle the -xrm option.
(defun x-handle-xrm-switch (switch)
  (or (consp x-invocation-args)
      (error "%s: missing argument to `%s' option" (invocation-name) switch))
  (setq x-command-line-resources (car x-invocation-args))
  (setq x-invocation-args (cdr x-invocation-args)))

;; Handle the geometry option
(defun x-handle-geometry (switch)
  (let ((geo (x-parse-geometry (car x-invocation-args))))
    (setq initial-frame-alist
	  (append initial-frame-alist
		  (if (or (assq 'left geo) (assq 'top geo))
		      '((user-position . t)))
		  (if (or (assq 'height geo) (assq 'width geo))
		      '((user-size . t)))
		  geo)
	  x-invocation-args (cdr x-invocation-args))))

;; Handle the -name and -rn options.  Set the variable x-resource-name
;; to the option's operand; if the switch was `-name', set the name of
;; the initial frame, too.
(defun x-handle-name-rn-switch (switch)
  (or (consp x-invocation-args)
      (error "%s: missing argument to `%s' option" (invocation-name) switch))
  (setq x-resource-name (car x-invocation-args)
	x-invocation-args (cdr x-invocation-args))
  (if (string= switch "-name")
      (setq initial-frame-alist (cons (cons 'name x-resource-name)
				      initial-frame-alist))))

(defvar x-display-name nil
  "The display name specifying server and frame.")

(defun x-handle-display (switch)
  (setq x-display-name (car x-invocation-args)
	x-invocation-args (cdr x-invocation-args)))

(defvar x-invocation-args nil)

(defun x-handle-args (args)
  "Process the X-related command line options in ARGS.
This is done before the user's startup file is loaded.  They are copied to
x-invocation args from which the X-related things are extracted, first
the switch (e.g., \"-fg\") in the following code, and possible values
\(e.g., \"black\") in the option handler code (e.g., x-handle-switch).
This returns ARGS with the arguments that have been processed removed."
  (message "%s" args)
  (setq x-invocation-args args
	args nil)
  (while x-invocation-args
    (let* ((this-switch (car x-invocation-args))
	   (orig-this-switch this-switch)
	   completion argval aelt)
      (setq x-invocation-args (cdr x-invocation-args))
      ;; Check for long options with attached arguments
      ;; and separate out the attached option argument into argval.
      (if (string-match "^--[^=]*=" this-switch)
	  (setq argval (substring this-switch (match-end 0))
		this-switch (substring this-switch 0 (1- (match-end 0)))))
      (setq completion (try-completion this-switch x-long-option-alist))
      (if (eq completion t)
	  ;; Exact match for long option.
	  (setq this-switch (cdr (assoc this-switch x-long-option-alist)))
	(if (stringp completion)
	    (let ((elt (assoc completion x-long-option-alist)))
	      ;; Check for abbreviated long option.
	      (or elt
		  (error "Option `%s' is ambiguous" this-switch))
	      (setq this-switch (cdr elt)))
	  ;; Check for a short option.
	  (setq argval nil this-switch orig-this-switch)))
      (setq aelt (assoc this-switch x-option-alist))
      (if aelt
	  (if argval
	      (let ((x-invocation-args
		     (cons argval x-invocation-args)))
		(funcall (cdr aelt) this-switch))
	    (funcall (cdr aelt) this-switch))
	(setq args (cons this-switch args)))))
  (setq args (nreverse args)))



;;
;; Available colors
;;

(defvar x-colors '("aquamarine"
		   "Aquamarine"
		   "medium aquamarine"
		   "MediumAquamarine"
		   "black"
		   "Black"
		   "blue"
		   "Blue"
		   "cadet blue"
		   "CadetBlue"
		   "cornflower blue"
		   "CornflowerBlue"
		   "dark slate blue"
		   "DarkSlateBlue"
		   "light blue"
		   "LightBlue"
		   "light steel blue"
		   "LightSteelBlue"
		   "medium blue"
		   "MediumBlue"
		   "medium slate blue"
		   "MediumSlateBlue"
		   "midnight blue"
		   "MidnightBlue"
		   "navy blue"
		   "NavyBlue"
		   "navy"
		   "Navy"
		   "sky blue"
		   "SkyBlue"
		   "slate blue"
		   "SlateBlue"
		   "steel blue"
		   "SteelBlue"
		   "coral"
		   "Coral"
		   "cyan"
		   "Cyan"
		   "firebrick"
		   "Firebrick"
		   "brown"
		   "Brown"
		   "gold"
		   "Gold"
		   "goldenrod"
		   "Goldenrod"
		   "green"
		   "Green"
		   "dark green"
		   "DarkGreen"
		   "dark olive green"
		   "DarkOliveGreen"
		   "forest green"
		   "ForestGreen"
		   "lime green"
		   "LimeGreen"
		   "medium sea green"
		   "MediumSeaGreen"
		   "medium spring green"
		   "MediumSpringGreen"
		   "pale green"
		   "PaleGreen"
		   "sea green"
		   "SeaGreen"
		   "spring green"
		   "SpringGreen"
		   "yellow green"
		   "YellowGreen"
		   "dark slate grey"
		   "DarkSlateGrey"
		   "dark slate gray"
		   "DarkSlateGray"
		   "dim grey"
		   "DimGrey"
		   "dim gray"
		   "DimGray"
		   "light grey"
		   "LightGrey"
		   "light gray"
		   "LightGray"
		   "gray"
		   "grey"
		   "Gray"
		   "Grey"
		   "khaki"
		   "Khaki"
		   "magenta"
		   "Magenta"
		   "maroon"
		   "Maroon"
		   "orange"
		   "Orange"
		   "orchid"
		   "Orchid"
		   "dark orchid"
		   "DarkOrchid"
		   "medium orchid"
		   "MediumOrchid"
		   "pink"
		   "Pink"
		   "plum"
		   "Plum"
		   "red"
		   "Red"
		   "indian red"
		   "IndianRed"
		   "medium violet red"
		   "MediumVioletRed"
		   "orange red"
		   "OrangeRed"
		   "violet red"
		   "VioletRed"
		   "salmon"
		   "Salmon"
		   "sienna"
		   "Sienna"
		   "tan"
		   "Tan"
		   "thistle"
		   "Thistle"
		   "turquoise"
		   "Turquoise"
		   "dark turquoise"
		   "DarkTurquoise"
		   "medium turquoise"
		   "MediumTurquoise"
		   "violet"
		   "Violet"
		   "blue violet"
		   "BlueViolet"
		   "wheat"
		   "Wheat"
		   "white"
		   "White"
		   "yellow"
		   "Yellow"
		   "green yellow"
		   "GreenYellow")
  "The full list of X colors from the `rgb.text' file.")

(defun x-defined-colors (&optional frame)
  "Return a list of colors supported for a particular frame.
The argument FRAME specifies which frame to try.
The value may be different for frames on different X displays."
  (or frame (setq frame (selected-frame)))
  (let ((all-colors x-colors)
	(this-color nil)
	(defined-colors nil))
    (while all-colors
      (setq this-color (car all-colors)
	    all-colors (cdr all-colors))
      (and (face-color-supported-p frame this-color t)
	   (setq defined-colors (cons this-color defined-colors))))
    defined-colors))

;;;; Function keys

(defun iconify-or-deiconify-frame ()
  "Iconify the selected frame, or deiconify if it's currently an icon."
  (interactive)
  (if (eq (cdr (assq 'visibility (frame-parameters))) t)
      (iconify-frame)
    (make-frame-visible)))

(substitute-key-definition 'suspend-emacs 'iconify-or-deiconify-frame
			   global-map)

;; Map certain keypad keys into ASCII characters
;; that people usually expect.
;(define-key function-key-map [backspace] [127])
;(define-key function-key-map [delete] [127])
(define-key function-key-map [tab] [?\t])
(define-key function-key-map [linefeed] [?\n])
(define-key function-key-map [clear] [11])
(define-key function-key-map [return] [13])
(define-key function-key-map [escape] [?\e])
;(define-key function-key-map [M-backspace] [?\M-\d])
;(define-key function-key-map [M-delete] [?\M-\d])
(define-key function-key-map [M-tab] [?\M-\t])
(define-key function-key-map [M-linefeed] [?\M-\n])
(define-key function-key-map [M-clear] [?\M-\013])
(define-key function-key-map [M-return] [?\M-\015])
(define-key function-key-map [M-escape] [?\M-\e])

;; These tell read-char how to convert
;; these special chars to ASCII.
;(put 'backspace 'ascii-character 127)
;(put 'delete 'ascii-character 127)
(put 'tab 'ascii-character ?\t)
(put 'linefeed 'ascii-character ?\n)
(put 'clear 'ascii-character 12)
(put 'return 'ascii-character 13)
(put 'escape 'ascii-character ?\e)


;;;; Selections and cut buffers

;;; We keep track of the last text selected here, so we can check the
;;; current selection against it, and avoid passing back our own text
;;; from x-cut-buffer-or-selection-value.
(defvar x-last-selected-text nil)

;;; It is said that overlarge strings are slow to put into the cut buffer.
;;; Note this value is overridden below.
(defvar x-cut-buffer-max 20000
  "Max number of characters to put in the cut buffer.")

(defvar x-select-enable-clipboard t
  "Non-nil means cutting and pasting uses the clipboard.
This is in addition to the primary selection.")

(defun x-select-text (text &optional push)
  (if x-select-enable-clipboard 
      (win32-set-clipboard-data text))
  (setq x-last-selected-text text))
    
;;; Return the value of the current selection.
;;; Consult the selection, then the cut buffer.  Treat empty strings
;;; as if they were unset.
(defun x-get-selection-value ()
  (if x-select-enable-clipboard 
      (let (text)
	;; Don't die if x-get-selection signals an error.
	(condition-case c
	    (setq text (win32-get-clipboard-data))
	  (error (message "win32-get-clipboard-data:%s" c)))
	(if (string= text "") (setq text nil))
	(cond
	 ((not text) nil)
	 ((eq text x-last-selected-text) nil)
	 ((string= text x-last-selected-text)
	  ;; Record the newer string, so subsequent calls can use the 'eq' test.
	  (setq x-last-selected-text text)
	  nil)
	 (t
	  (setq x-last-selected-text text))))))

;;; Do the actual Windows setup here; the above code just defines
;;; functions and variables that we use now.

(setq command-line-args (x-handle-args command-line-args))

;;; Make sure we have a valid resource name.
(or (stringp x-resource-name)
    (let (i)
      (setq x-resource-name (invocation-name))

      ;; Change any . or * characters in x-resource-name to hyphens,
      ;; so as not to choke when we use it in X resource queries.
      (while (setq i (string-match "[.*]" x-resource-name))
	(aset x-resource-name i ?-))))

;; For the benefit of older Emacses (19.27 and earlier) that are sharing
;; the same lisp directory, don't pass the third argument unless we seem
;; to have the multi-display support.
(if (fboundp 'x-close-connection)
    (x-open-connection ""
		       x-command-line-resources
		       ;; Exit Emacs with fatal error if this fails.
		       t)
  (x-open-connection ""
		     x-command-line-resources))

(setq frame-creation-function 'x-create-frame-with-faces)

(setq x-cut-buffer-max (min (- (/ (x-server-max-request-size) 2) 100)
			    x-cut-buffer-max))

;; Win32 expects the menu bar cut and paste commands to use the clipboard.
;; This has ,? to match both on Sunos and on Solaris.
(menu-bar-enable-clipboard)

;; Apply a geometry resource to the initial frame.  Put it at the end
;; of the alist, so that anything specified on the command line takes
;; precedence.
(let* ((res-geometry (x-get-resource "geometry" "Geometry"))
       parsed)
  (if res-geometry
      (progn
	(setq parsed (x-parse-geometry res-geometry))
	;; If the resource specifies a position,
	;; call the position and size "user-specified".
	(if (or (assq 'top parsed) (assq 'left parsed))
	    (setq parsed (cons '(user-position . t)
			       (cons '(user-size . t) parsed))))
	;; All geometry parms apply to the initial frame.
	(setq initial-frame-alist (append initial-frame-alist parsed))
	;; The size parms apply to all frames.
	(if (assq 'height parsed)
	    (setq default-frame-alist
		  (cons (cons 'height (cdr (assq 'height parsed)))
			default-frame-alist)))
	(if (assq 'width parsed)
	    (setq default-frame-alist
		  (cons (cons 'width (cdr (assq 'width parsed)))
			default-frame-alist))))))

;; Check the reverseVideo resource.
(let ((case-fold-search t))
  (let ((rv (x-get-resource "reverseVideo" "ReverseVideo")))
    (if (and rv
	     (string-match "^\\(true\\|yes\\|on\\)$" rv))
	(setq default-frame-alist
	      (cons '(reverse . t) default-frame-alist)))))

;; Set x-selection-timeout, measured in milliseconds.
(let ((res-selection-timeout
       (x-get-resource "selectionTimeout" "SelectionTimeout")))
  (setq x-selection-timeout 20000)
  (if res-selection-timeout
      (setq x-selection-timeout (string-to-number res-selection-timeout))))

(defun x-win-suspend-error ()
  (error "Suspending an emacs running under Win32 makes no sense"))
(add-hook 'suspend-hook 'x-win-suspend-error)

;;; Arrange for the kill and yank functions to set and check the clipboard.
(setq interprogram-cut-function 'x-select-text)
(setq interprogram-paste-function 'x-get-selection-value)

;;; Turn off window-splitting optimization; win32 is usually fast enough
;;; that this is only annoying.
(setq split-window-keep-point t)

;; Don't show the frame name; that's redundant.
(setq-default mode-line-buffer-identification
	      (if (boundp 'MULE)
		  '("Mule: %12b")
		'("Emacs: %12b")))

;; Remap some functions to call win32 common dialogs

(defun internal-face-interactive (what &optional bool)
  (let* ((fn (intern (concat "face-" what)))
	 (prompt (concat "Set " what " of face"))
	 (face (read-face-name (concat prompt ": ")))
	 (default (if (fboundp fn)
		      (or (funcall fn face (selected-frame))
			  (funcall fn 'default (selected-frame)))))
	 (fn-win (intern (concat (symbol-name window-system) "-select-" what)))
	 (value 
	  (if (fboundp fn-win)
	      (funcall fn-win)
	    (if bool
		(y-or-n-p (concat "Should face " (symbol-name face)
				  " be " bool "? "))
	      (read-string (concat prompt " " (symbol-name face) " to: ")
			   default)))))
	 (list face (if (equal value "") nil value))))

;; Redefine the font selection to use the Win32 dialog

(or (boundp 'win32-fontset)
    (progn
(defun list-fonts-internal (face font &optional frame)
  "Change the font of face FACE to FONT (a string).
If the optional FRAME argument is provided, change only
in that frame; otherwise change each frame."
  (interactive (internal-face-interactive "font"))
  (win32-list-fonts font face frame))

;; The name list-faces would be more consistent, but let's avoid a conflict
;; with Lucid, which uses that name differently.
(defun list-fonts (face font &optional frame)
  "List all fonts."
  (interactive (internal-face-interactive "font"))
  (let ((fonts (sort (win32-list-fonts font face frame) (function string-lessp)))
	(font nil)
	(frame (selected-frame))
	disp-frame window)
    (with-output-to-temp-buffer "*Fonts*"
      (save-excursion
	(set-buffer standard-output)
	(setq truncate-lines t)
	(while fonts
	  (setq font (car fonts))
	  (setq fonts (cdr fonts))
	  (insert (format "%25s " font))
	  (let ((beg (point)))
	    (insert "\n")
	    (goto-char beg)
	    (forward-line 1)
	    (while (not (eobp))
	      (insert "                          ")
	      (forward-line 1))))
	(goto-char (point-min))))
    ;; If the *Fonts* buffer appears in a different frame,
    ;; copy all the font definitions from FRAME,
    ;; so that the display will reflect the frame that was selected.
    (setq window (get-buffer-window (get-buffer "*Fonts*") t))
    (setq disp-frame (if window (window-frame window)
		       (car (frame-list))))
    (or (eq frame disp-frame)
	(let ((fonts (win32-list-fonts font face frame)))
	  (while fonts
	    (copy-font (car fonts) (car fonts) frame disp-frame)
	    (setq fonts (cdr fonts)))))))
))

;;; Turn off window-splitting optimization; X is usually fast enough
;;; that this is only annoying.
(setq split-window-keep-point t)

;; Set up to recognize vendor-specific keysyms.
;; Unless/until there is a real conflict,
;; we need not try to make this list depend on
;; the type of X server in use.
;(setq system-key-alist
;      '(
;	;; These are some HP keys.
;	(  168 . mute-acute)
;	(  169 . mute-grave)
;	(  170 . mute-asciicircum)
;	(  171 . mute-diaeresis)
;	(  172 . mute-asciitilde)
;	(  175 . lira)
;	(  190 . guilder)
;	(  252 . block)
;	(  256 . longminus)
;	(65388 . reset)
;	(65389 . system)
;	(65390 . user)
;	(65391 . clearline)
;	(65392 . insertline)
;	(65393 . deleteline)
;	(65394 . insertchar)
;	(65395 . deletechar)
;	(65396 . backtab)
;	(65397 . kp-backtab)
;	;; This is used by DEC's X server.
;	(65280 . remove)
;	;; These are for Sun.
;	(392976 . f35)
;    	(392977 . f36)
;	(393056 . req)
;	;; These are for Sun under X11R6
;	(393072 . props)
;	(393073 . front)
;	(393074 . copy)
;	(393075 . open)
;	(393076 . paste)
;	(393077 . cut)
;	))
;
;;

;; set-up language-environment and fontset.
;;
(let* ((win32-version-4 (= (nth 0 (win32-os-version-info)) 4))
       (win32-winnt (eq (win32-platform-id) 'nt))
       (lang-info (win32-setup-language-environment))
       (win32-user-language-environment (cdr lang-info))
       (lang-name (car lang-info)))

  ;; set-up Latin-1 fonts.
  (nconc default-fonts-12
	 '(((lc-ltn1 . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) latin-1 49))
	    "default-ltn1-12")))
  (nconc default-fonts-14
	 '(((lc-ltn1 . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) latin-1 49))
	    "default-ltn1-14")))
  (nconc default-fonts-16
	 '(((lc-ltn1 . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) latin-1 49))
	    "default-ltn1-16")))

  ;; set-up Latin-2 fonts.
  (cond ((or (eq win32-user-language-environment 'win-latin2)
	     win32-version-4)	;; for Windows95 and Windows NT 4.0
	 (nconc default-fonts-12
		'(((lc-ltn2 . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) latin-2 49))
		   "default-ltn2-12")))
	 (nconc default-fonts-14
		'(((lc-ltn2 . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) latin-2 49))
		   "default-ltn2-14")))
	 (nconc default-fonts-16
		'(((lc-ltn2 . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) latin-2 49))
		   "default-ltn2-16"))))
	((win32-font-exist-p "Latin2 sMTF")   ;; for Windows NT 3.0 and NT 3.51
	 (nconc default-fonts-12
		'(((lc-ltn2 . ("Latin2 sMTF" (0 0 0 0) (12 12 11 11) symbol 49))
		   "default-ltn2-12")))
	 (nconc default-fonts-14
		'(((lc-ltn2 . ("Latin2 sMTF" (0 0 0 0) (14 14 12 12) symbol 49))
		   "default-ltn2-14")))
	 (nconc default-fonts-16
		'(((lc-ltn2 . ("Latin2 sMTF" (0 0 0 0) (16 16 14 14) symbol 49))
		   "default-ltn2-16"))))
	(t
	 (and (win32-font-exist-p "Latin2 MBF12")
	      (nconc default-fonts-12
		     '(((lc-ltn2 . ("Latin2 MBF12" 6 12 ansi 1))
			"default-ltn2-12"))))
	 (and (win32-font-exist-p "Latin2 MBF14")
	      (nconc default-fonts-14
		     '(((lc-ltn2 . ("Latin2 MBF14" 7 14 ansi 1))
			"default-ltn2-14"))))
	 (and (win32-font-exist-p "Latin2 MBF16")
	      (nconc default-fonts-16
		     '(((lc-ltn2 . ("Latin2 MBF16" 8 16 ansi 1))
			"default-ltn2-16"))))))

  ;; setup Greek fonts.
  (cond ((or (eq win32-user-language-environment 'greek)
	     win32-version-4)	;; for Windows95 and Windows NT 4.0
	 (nconc default-fonts-12
		'(((lc-grk . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) Greek 49))
		   "default-grk-12")))
	 (nconc default-fonts-14
		'(((lc-grk . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) Greek 49))
		   "default-grk-14")))
	 (nconc default-fonts-16
		'(((lc-grk . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) Greek 49))
		   "default-grk-16"))))
	((win32-font-exist-p "Greek MTF")  ;; for Windows NT 3.0 and NT 3.51
	 (nconc default-fonts-12
		'(((lc-grk . ("Greek MTF" (0 0 0 0) (12 12 11 11) ansi 49))
		   "default-grk-12")))
	 (nconc default-fonts-14
		'(((lc-grk . ("Greek MTF" (0 0 0 0) (14 14 12 12) ansi 49))
		   "default-grk-14")))
	 (nconc default-fonts-16
		'(((lc-grk . ("Greek MTF" (0 0 0 0) (16 16 14 14) ansi 49))
		   "default-grk-16"))))
	(t
	 (and (win32-font-exist-p "Greek MBF12")
	      (nconc default-fonts-12
		     '(((lc-grk . ("Greek MBF12" 7 12 ansi 1))
			"default-grk-12"))))
	 (and (win32-font-exist-p "Greek MBF14")
	      (nconc default-fonts-14
		     '(((lc-grk . ("Greek MBF14" 7 14 ansi 17))
			"default-grk-14"))))
	 (and (win32-font-exist-p "Greek MBF16")
	      (nconc default-fonts-16
		     '(((lc-grk . ("Greek MBF16" 8 16 ansi 17))
			"default-grk-16"))))))

  ;; set-up Cyrillic fonts.
  (cond ((or (eq win32-user-language-environment 'win-cyrillic)
	     win32-version-4)	;; for Windows95 and Windows NT 4.0
	 (nconc default-fonts-12
		'(((lc-crl . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) Cyrillic 49))
		   "default-crl-12")))
	 (nconc default-fonts-14
		'(((lc-crl . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) Cyrillic 49))
		   "default-crl-14")))
	 (nconc default-fonts-16
		'(((lc-crl . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) Cyrillic 49))
		   "default-crl-16"))))
	((win32-font-exist-p "Cyrillic sMTF") ;; for Windows NT 3.0 and NT 3.51
	 (nconc default-fonts-12
		'(((lc-crl . ("Cyrillic sMTF" (0 0 0 0) (12 12 11 11) symbol 49))
		   "default-crl-12")))
	 (nconc default-fonts-14
		'(((lc-crl . ("Cyrillic sMTF" (0 0 0 0) (14 14 12 12) symbol 49))
		   "default-crl-14")))
	 (nconc default-fonts-16
		'(((lc-crl . ("Cyrillic sMTF" (0 0 0 0) (16 16 14 14) symbol 49))
		   "default-crl-16"))))
	(t
	 (and (win32-font-exist-p "Cyrillic MBF12")
	      (nconc default-fonts-12
		     '(((lc-crl . ("Cyrillic MBF12" 6 12 ansi 17))
			"default-crl-12"))))
	 (and (win32-font-exist-p "Cyrillic MBF14")
	      (nconc default-fonts-14
		     '(((lc-crl . ("Cyrillic MBF14" 7 14 ansi 17))
			"default-crl-14"))))
	 (and (win32-font-exist-p "Cyrillic MBF16")
	      (nconc default-fonts-16
		     '(((lc-crl . ("Cyrillic MBF16" 8 16 ansi 17))
			"default-crl-16"))))))

  ;; set-up Turkish fonts.
  (cond ((or (eq lang-name 'trk)
	     win32-version-4)	;; for Windows95 and Windows NT 4.0
	 (nconc default-fonts-12
		'(((lc-ltn5 . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) Turkish 49))
		   "default-trk-12")))
	 (nconc default-fonts-14
		'(((lc-ltn5 . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) Turkish 49))
		   "default-trk-14")))
	 (nconc default-fonts-16
		'(((lc-ltn5 . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) Turkish 49))
		   "default-trk-16"))))
	((win32-font-exist-p "Latin9 MTF")  ;; for Windows NT 3.0 and NT 3.51
	 (nconc default-fonts-12
		'(((lc-ltn5 . ("Latin9 MTF" (0 0 0 0) (12 12 11 11) ansi 49))
		   "default-trk-12")))
	 (nconc default-fonts-14
		'(((lc-ltn5 . ("Latin9 MTF" (0 0 0 0) (14 14 12 12) ansi 49))
		   "default-trk-14")))
	 (nconc default-fonts-16
		'(((lc-ltn5 . ("Latin9 MTF" (0 0 0 0) (16 16 14 14) ansi 49))
		   "default-trk-16"))))
	(t
	 (and (win32-font-exist-p "Turkish MBF12")
	      (nconc default-fonts-12
		     '(((lc-ltn5 . ("Turkish MBF12" 6 12 ansi 17))
			"default-trk-12"))))
	 (and (win32-font-exist-p "Turkish MBF14")
	      (nconc default-fonts-14
		     '(((lc-ltn5 . ("Turkish MBF14" 7 14 ansi 17))
			"default-trk-14"))))
	 (and (win32-font-exist-p "Turkish MBF16")
	      (nconc default-fonts-16
		     '(((lc-ltn5 . ("Turkish MBF16" 8 16 ansi 17))
			"default-trk-16"))))))

  ;; set-up Hebrew Fonts.
  (cond ((and win32-version-4
	      win32-winnt)	;; for Windows NT 4.0
	 (nconc default-fonts-12
		'(((lc-hbw . ("Lucida Sans Unicode" (0 0 0 0) (-10 -10 -9 -9) Hebrew 49))
		   "default-hbw-12")
		  ((lc-ascr2l . ("Lucida Sans Unicode" (0 0 0 0) (-10 -10 -9 -9) ansi 49))
		   "default-ascr2l-12")))
	 (nconc default-fonts-14
		'(((lc-hbw . ("Lucida Sans Unicode" (0 0 0 0) (-11 -11 -10 -10) Hebrew 49))
		   "default-hbw-14")
		  ((lc-ascr2l . ("Lusida Sans Unicode" (0 0 0 0) (-11 -11 -10 -10) ansi 49))
		   "default-ascr2l-14")))
	 (nconc default-fonts-16
		'(((lc-hbw . ("Lucida Sans Unicode" (0 0 0 0) (-12 -12 -11 -11) Hebrew 49))
		   "default-hbw-16")
		  ((lc-ascr2l . ("Lucida Sans Unicode" (0 0 0 0) (-12 -12 -11 -11) ansi 49))
		   "default-ascr2l-16"))))
	((win32-font-exist-p "Hebrew MTF")  ;; for Windows95, Windows NT 3.0 and NT 3.51
	 (nconc default-fonts-12
		'(((lc-hbw . ("Hebrew MTF" (0 0 0 0) (12 12 11 11) ansi 49))
		   "default-hbw-12")
		  ((lc-ascr2l . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) ansi 49))
		   "default-ascr2l-12")))
	 (nconc default-fonts-14
		'(((lc-hbw . ("Hebrew MTF" (0 0 0 0) (14 14 12 12) ansi 49))
		   "default-hbw-14")
		  ((lc-ascr2l . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) ansi 49))
		   "default-ascr2l-14")))
	 (nconc default-fonts-16
		'(((lc-hbw . ("Hebrew MTF" (0 0 0 0) (16 16 14 14) ansi 49))
		   "default-hbw-16")
		  ((lc-ascr2l . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) ansi 49))
		   "default-ascr2l-16"))))
	(t
	 (and (win32-font-exist-p "Hebrew MBF12")
	      (nconc default-fonts-12
		     '(((lc-hbw . ("Hebrew MBF12" 6 12 ansi 17))
			"default-hbw-12")
		       ((lc-ascr2l . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) ansi 49))
			"default-ascr2l-12"))))
	 (and (win32-font-exist-p "Hebrew MBF14")
	      (nconc default-fonts-14
		     '(((lc-hbw . ("Hebrew MBF14" 7 14 ansi 17))
			"default-hbw-14")
		       ((lc-ascr2l . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) ansi 49))
			"default-ascr2l-14"))))
	 (and (win32-font-exist-p "Hebrew MBF16")
	      (nconc default-fonts-16
		     '(((lc-hbw . ("Hebrew MBF16" 8 16 ansi 17))
			"default-hbw-16")
		       ((lc-ascr2l . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) ansi 49))
			"default-ascr2l-16"))))))

  ;; set-up Japanese fonts.
  (if (eq win32-user-language-environment 'japanese)
      (progn
	(nconc default-fonts-12
	       '(((lc-jp . ("$B#M#S(B $B%4%7%C%/(B" (0 0 0 0) (-12 -12 -11 -11) Japanese 49))
		  "default-jp-12")))
	(nconc default-fonts-14
	       '(((lc-jp . ("$B#M#S(B $B%4%7%C%/(B" (0 0 0 0) (-14 -14 -13 -13) Japanese 49))
		  "default-jp-14")))
	(nconc default-fonts-16
	       '(((lc-jp . ("$B#M#S(B $B%4%7%C%/(B" (0 0 0 0) (-16 -16 -15 -15) Japanese 49))
		  "default-jp-16")))
	)
    (and win32-version-4
	 (win32-font-exist-p "MS Gothic")
	 (nconc default-fonts-12
		'(((lc-jp . ("MS Gothic" (0 0 0 0) (-12 -12 -11 -11) Japanese 49))
		   "default-jp-12")))
	 (nconc default-fonts-14
		'(((lc-jp . ("MS Gothic" (0 0 0 0) (-14 -14 -13 -13) Japanese 49))
		   "default-jp-14")))
	 (nconc default-fonts-16
		'(((lc-jp . ("MS Gothic" (0 0 0 0) (-16 -16 -15 -15) Japanese 49))
		   "default-jp-16")))))

  ;; set-up Chinese BIG5 fonts.
  (and (or (eq lang-name 'cht)
	   (win32-font-exist-p "MingLiU"))
       (nconc default-fonts-12
	      '(((lc-big5-1 . ("MingLiU" (0 0 0 0) (-12 -12 -11 -11) big5 49))
		 "default-big5-12")))
       (nconc default-fonts-14
	      '(((lc-big5-1 . ("MingLiU" (0 0 0 0) (-14 -14 -13 -13) big5 49))
		 "default-big5-14")))
       (nconc default-fonts-16
	      '(((lc-big5-1 . ("MingLiU" (0 0 0 0) (-16 -16 -15 -15) big5 49))
		 "default-big5-16"))))

  ;; set-up Chinese GB2312 fonts.
  (cond ((or (eq lang-name 'chs)
	     (win32-font-exist-p "MS Song"))
	 (nconc default-fonts-12
		'(((lc-cn . ("MS Song" (0 0 0 0) (-12 -12 -11 -11) gd2312 49))
		   "default-gb-12")))
	 (nconc default-fonts-14
		'(((lc-cn . ("MS Song" (0 0 0 0) (-14 -14 -13 -13) gb2312 49))
		   "default-gb-14")))
	 (nconc default-fonts-16
		'(((lc-cn . ("MS Song" (0 0 0 0) (-16 -16 -15 -15) gb2312 49))
		   "default-gb-16"))))
	((win32-font-exist-p "MS Hei")
	 (nconc default-fonts-12
		'(((lc-cn . ("MS Hei" (0 0 0 0) (-12 -12 -11 -11) gb2312 49))
		   "default-gb-12")))
	 (nconc default-fonts-14
		'(((lc-cn . ("MS Hei" (0 0 0 0) (-14 -14 -13 -13) gb2312 49))
		   "default-gb-14")))
	 (nconc default-fonts-16
		'(((lc-cn . ("MS Hei" (0 0 0 0) (-16 -16 -15 -15) gb2312 49))
		   "default-gb-16"))))
	(t
	 nil))

  ;; set-up Korean fonts.
  (cond ((or (eq win32-user-language-environment 'korean)
	     (win32-font-exist-p "Gulim"))
	 (nconc default-fonts-12
		'(((lc-kr . ("Gulim" (0 0 0 0) (-12 -12 -11 -11) Korean 49))
		   "default-kr-12")))
	 (nconc default-fonts-14
		'(((lc-kr . ("Gulim" (0 0 0 0) (-14 -14 -13 -13) Korean 49))
		   "default-kr-14")))
	 (nconc default-fonts-16
		'(((lc-kr . ("Gulim" (0 0 0 0) (-16 -16 -15 -15) Korean 49))
		   "default-kr-16"))))
	((win32-font-exist-p "GulimChe")
	 (nconc default-fonts-12
		'(((lc-kr . ("GulimChe" (0 0 0 0) (-12 -12 -11 -11) Korean 49))
		   "default-kr-12")))
	 (nconc default-fonts-14
		'(((lc-kr . ("GulimChe" (0 0 0 0) (-14 -14 -13 -13) Korean 49))
		   "default-kr-14")))
	 (nconc default-fonts-16
		'(((lc-kr . ("GulimChe" (0 0 0 0) (-16 -16 -15 -15) Korean 49))
		   "default-kr-16"))))
	(t
	 nil))

  ;; setup MuleTrueTypeFont or MuleBitmapFont Fonts.
  (cond ((win32-font-exist-p "Latin3 MTF")
	 (nconc default-fonts-12
		'(((lc-ltn3 . ("Latin3 MTF" (0 0 0 0) (12 12 11 11) ansi 49))
		   "default-ltn3-12")))
	 (nconc default-fonts-14
		'(((lc-ltn3 . ("Latin3 MTF" (0 0 0 0) (14 14 12 12) ansi 49))
		   "default-ltn3-14")))
	 (nconc default-fonts-16
		'(((lc-ltn3 . ("Latin3 MTF" (0 0 0 0) (16 16 14 14) ansi 49))
		   "default-ltn3-16"))))
	(t
	 (and (win32-font-exist-p "Latin3 MBF12")
	      (nconc default-fonts-12
		     '(((lc-ltn3 . ("Latin3 MBF12" 6 12 ansi 17))
			"default-ltn3-12"))))
	 (and (win32-font-exist-p "Latin3 MBF14")
	      (nconc default-fonts-14
		     '(((lc-ltn3 . ("Latin3 MBF14" 7 14 ansi 17))
			"default-ltn3-14"))))
	 (and (win32-font-exist-p "Latin3 MBF16")
	      (nconc default-fonts-16
		     '(((lc-ltn3 . ("Latin3 MBF16" 8 16 ansi 17))
			"default-ltn3-16"))))))

  (cond ((win32-font-exist-p "Latin4 MTF")
	 (nconc default-fonts-12
		'(((lc-ltn4 . ("Latin4 MTF" (0 0 0 0) (12 12 11 11) ansi 49))
		   "default-ltn4-12")))
	 (nconc default-fonts-14
		'(((lc-ltn4 . ("Latin4 MTF" (0 0 0 0) (14 14 12 12) ansi 49))
		   "default-ltn4-14")))
	 (nconc default-fonts-16
		'(((lc-ltn4 . ("Latin4 MTF" (0 0 0 0) (16 16 14 14) ansi 49))
		   "default-ltn4-16"))))
	(t
	 (and (win32-font-exist-p "Latin4 MBF12")
	      (nconc default-fonts-12
		     '(((lc-ltn4 . ("Latin4 MBF12" 6 12 ansi 17))
			"default-ltn4-12"))))
	 (and (win32-font-exist-p "Latin4 MBF14")
	      (nconc default-fonts-14
		     '(((lc-ltn4 . ("Latin4 MBF14" 7 14 ansi 17))
			"default-ltn4-14"))))
	 (and (win32-font-exist-p "Latin4 MBF16")
	      (nconc default-fonts-16
		     '(((lc-ltn4 . ("Latin4 MBF16" 8 16 ansi 17))
			"default-ltn4-16"))))))

  (cond ((win32-font-exist-p "Mule IPA sMTF")
	 (nconc default-fonts-12
		'(((lc-ipa0 . ("Mule IPA sMTF" (0 0 0 0) (12 12 11 11) symbol 49))
		   "default-ipa0-12")))
	 (nconc default-fonts-14
		'(((lc-ipa0 . ("Mule IPA sMTF" (0 0 0 0) (14 14 12 12) symbol 49))
		   "default-ipa0-14")))
	 (nconc default-fonts-16
		'(((lc-ipa0 . ("Mule IPA sMTF" (0 0 0 0) (16 16 14 14) symbol 49))
		   "default-ipa0-16"))))
	(t
	 (and (win32-font-exist-p "IPA ETL MBF12")
	      (nconc default-fonts-12
		     '(((lc-ipa0 . ("IPA ETL MBF12" 6 12 ansi 1))
			"default-ipa0-12"))))
	 (and (win32-font-exist-p "IPA ETL MBF14")
	      (nconc default-fonts-14
		     '(((lc-ipa0 . ("IPA ETL MBF14" 7 14 ansi 1))
			"default-ipa0-14"))))
	 (and (win32-font-exist-p "IPA ETL MBF16")
	      (nconc default-fonts-16
		     '(((lc-ipa0 . ("IPA ETL MBF16" 8 16 ansi 1))
			"default-ipa0-16"))))))

  (cond ((win32-font-exist-p "Thai Pen sMTF")
	 (nconc default-fonts-12
		'(((lc-thai . ("Thai Pen sMTF" (0 0 0 0) (12 12 11 11) symbol 49))
		   "default-thai-12")))
	 (nconc default-fonts-14
		'(((lc-thai . ("Thai Pen sMTF" (0 0 0 0) (14 14 12 12) symbol 49))
		   "default-thai-14")))
	 (nconc default-fonts-16
		'(((lc-thai . ("Thai Pen sMTF" (0 0 0 0) (16 16 14 14) symbol 49))
		   "default-thai-16"))))
	(t
	 (and (win32-font-exist-p "Tis620 ETL12")
	      (nconc default-fonts-12
		     '(((lc-thai . ("Tis620 ETL12" 6 12 ansi 1))
			"default-thai-12"))))
	 (and (win32-font-exist-p "Tis620 ETL14")
	      (nconc default-fonts-14
		     '(((lc-thai . ("Tis620 ETL14" 7 14 ansi 1))
			"default-that-14"))))
	 (and (win32-font-exist-p "Tis620 ETL16")
	      (nconc default-fonts-16
		     '(((lc-thai . ("Tis620 ETL16" 8 16 ansi 1))
			"default-thai-16"))))))

  (cond ((win32-font-exist-p "VISCII sMTF")
	 (nconc default-fonts-12
		'(((lc-vn-1 . ("VISCII sMTF" (0 0 0 0) (-11 -11 -10 -10) symbol 49))
		   "default-vn-1-12")
		  ((lc-vn-2 . ("VISCII sMTF" (0 0 0 0) (-11 -11 -10 -10) symbol 49))
		   "default-vn-2-12")))
	 (nconc default-fonts-14
		'(((lc-vn-1 . ("VISCII sMTF" (0 0 0 0) (-13 -13 -11 -11) symbol 49))
		   "default-vn-1-14")
		  ((lc-vn-2 . ("VISCII sMTF" (0 0 0 0) (-13 -13 -11 -11) symbol 49))
		   "default-vn-2-14")))
	 (nconc default-fonts-16
		'(((lc-vn-1 . ("VISCII sMTF" (0 0 0 0) (-15 -15 -12 -12) symbol 49))
		   "default-vn-1-16")
		  ((lc-vn-2 . ("VISCII sMTF" (0 0 0 0) (-15 -15 -12 -12) symbol 49))
		   "default-vn-2-16"))))
	(t
	 (and (win32-font-exist-p "VISCII-1 MBF12")
	      (nconc default-fonts-12
		     '(((lc-vn-1 . ("VISCII-1 MBF12" 7 12 ansi 1))
			"default-vn-1-12")
		       ((lc-vn-2 . ("VISCII-1 MBF12" 7 12 ansi 1))
			"default-vn-2-12"))))
	 (and (win32-font-exist-p "VISCII-1 MBF14")
	      (nconc default-fonts-14
		     '(((lc-vn-1 . ("VISCII-1 MBF14" 7 15 ansi 1))
			"default-vn-1-14")
		       ((lc-vn-2 . ("VISCII-1 MBF14" 7 15 ansi 1))
			"default-vn-2-14"))))
	 (and (win32-font-exist-p "VISCII ETL MBF16")
	      (nconc default-fonts-16
		     '(((lc-vn-1 . ("VISCII ETL MBF16" 8 16 ansi 1))
			"default-vn-1-16")
		       ((lc-vn-2 . ("VISCII ETL MBF16" 8 16 ansi 1))
			"default-vn-2-16"))))))

  (and (win32-font-exist-p "Mule Arabic0 ETL MBF12")
       (nconc default-fonts-12
	      '(((lc-arb0 . ("Mule Arabic0 ETL MBF12" 6 18 ansi 1))
		 "default-arb0-12"))))
  (and (win32-font-exist-p "Mule Arabic1 ETL MBF12")
       (nconc default-fonts-12
	      '(((lc-arb1 . ("Mule Arabic1 ETL MBF12" 6 18 ansi 1))
		 "default-arb1-12"))))
  (and (win32-font-exist-p "Mule Arabic2 ETL MBF12")
       (nconc default-fonts-12
	      '(((lc-arb2 . ("Mule Arabic2 ETL MBF12" 12 18 ansi 1))
		 "default-arb2-12"))))

  (and (win32-font-exist-p "Mule Arabic0 ETL MBF14")
       (nconc default-fonts-14
	      '(((lc-arb0 . ("Mule Arabic0 ETL MBF14" 7 21 ansi 1))
		 "default-arb0-14"))))
  (and (win32-font-exist-p "Mule Arabic1 ETL MBF14")
       (nconc default-fonts-14
	      '(((lc-arb1 . ("Mule Arabic1 ETL MBF14" 7 21 ansi 1))
		 "default-arb1-14"))))
  (and (win32-font-exist-p "Mule Arabic2 ETL MBF14")
       (nconc default-fonts-14
	      '(((lc-arb2 . ("Mule Arabic2 ETL MBF14" 14 21 ansi 1))
		 "default-arb2-14"))))

  (and (win32-font-exist-p "Mule Arabic0 ETL MBF16")
       (nconc default-fonts-16
	      '(((lc-arb0 . ("Mule Arabic0 ETL MBF16" 8 25 ansi 1))
		 "default-arb0-16"))))
  (and (win32-font-exist-p "Mule Arabic1 ETL MBF16")
       (nconc default-fonts-16
	      '(((lc-arb1 . ("Mule Arabic1 ETL MBF16" 8 25 ansi 1))
		 "default-arb1-16"))))
  (and (win32-font-exist-p "Mule Arabic2 ETL MBF16")
       (nconc default-fonts-16
	      '(((lc-arb2 . ("Mule Arabic2 ETL MBF16" 16 25 ansi 1))
		 "default-arb2-16"))))
  ;; end of setup Mule TrueTypeFonts.

  (win32-new-fontset "default" default-fonts-16 16)
)

;;; Set to a system sound if you want a fancy bell.
(set-message-beep 'visible)		; visible-bell

;(defun mouse-set-font (&rest fonts)
;  (interactive)
;  (set-default-font (win32-select-font)))

;;; win32-win.el ends here
